%  Call the subroutine of  ZIGEEfit to fit zero-inflated binomial (ZIB) model
%  or zero-inflated Poisson (ZIP) model
%  to the two response variables in the whitefly data


% Define global variables which can be used in subroutines

global YVAR LINKFUNCTION XVARS ERRDISFUNCTION SCALEPAR INC_CONST;
global OFFSET WEIGHTS DETAILSFILE NAMEXV NAMELIST NAMEYV;
global Y ID ID2 N2 N BMAT GMAT BINN2 ERRDIST2 MU2LINK PLINK CORSTRUCT MIXPCORSTR;
global VARFUNCTION; 


path(path,'../program')
load 'whitefly.dat';             %load the data set
ID=whitefly(:,7);
N=size(ID,1);
N2=max(ID);
Y=whitefly(:,6);
week=whitefly(:,2);
rep=whitefly(:,3);
trt=whitefly(:,4);
BINN=whitefly(:,5);
trtmat=fac(trt,6);      %use the function of fac to makes a matrix of indicators for 
                        %all treatment levels except trt=6
                   
weekmat=fac(week);      %makes a matrix of indicators for all weeks except week=1
repmat=fac(rep);   

ID2=zeros(N,1);
for i=1:N2;
  ID2(ID==i)=[1:size(ID(ID==i),1)]';
end;

BMAT=[ones(N,1), repmat, trtmat, week];      % covariate matrix used in modelling mean
GMAT=[ones(N,1)];                            % covariate matrix used in modelling mixing probabilities


% set initial values of parameters

betainit=[-0.655 0.385 0.315 -0.745 -0.368 -0.721 -0.313 2.556 0]';
gammainit=[-1.1]';
alphabinit=0;
alphaginit=0;
phiinit=1;

glmlab;

model='binomial';

% for ZIB model

if (strcmp(model,'binomial'))
PLINK='logit';                   %define link function for mixing probabilities 
MU2LINK='logit';                 %define link function for "sucess" probability in binomial Dist.
ERRDIST2='binoml';
BINN2=BINN;
VARFUNCTION='binomial';
end;

% for ZIP model

if (strcmp(model,'poisson'))
PLINK='logit';
MU2LINK='log';
ERRDIST2='poisson';
BINN2=ones(N,1);
VARFUNCTION='identity';
end;

CORSTRUCT='exch';             %define working correlation structure
MIXPCORSTR='indep';           %define working correlation structure for mixing mechanism 


diary whitefly.diary

%%%%%%%%%call the subroutine of ZIGEEfit to fit data using the ES algorithm incorprating GEE
[beta,gamma,alphab,alphag,phi]=ZIGEEfit(gammainit,betainit,alphabinit,alphaginit,phiinit)

diary off

