% Calculates numerical derivative of s in equation of (3.7)
% used in calculating variance and covariance of estimated parameters
% this function needs to call the function of getSandVnew()
% called by ZIGEEfit   


function [score]=getdsdparm(s,gamma, beta, alpha_g, alpha_b, phi)


global ID ID2 Y XMAT ZMAT WMAT N BINN1 BINN2 ERRDIST1 ERRDIST2 N2;
global MU1LINK MU2LINK PLINK MIXPCORSTR CORSTRUCT;


dimgamma=size(gamma,1);
dimbeta=size(beta,1);
if (strcmp(MIXPCORSTR,'indep'))
  dimalpha_g=0;
else
  dimalpha_g=size(alpha_g,1);
end;
if (strcmp(CORSTRUCT,'indep'))
  dimalpha_b=0;
else
  dimalpha_b=size(alpha_b,1);
end; 
dimparm=dimgamma+dimbeta+dimalpha_g+dimalpha_b+1;   %dimension of parameters

ef=2.2204e-16;         % machine precision number used in numerical derivatives (see SAS) 
score=zeros(dimparm,dimparm);

for i=1:dimparm;       

   %%%%%%%%%%%%% numerical derivative wrt gammas

    for j=1:dimgamma     

    x=gamma(j);

    if (x~=0)
      h=ef^(1/2)*(1+abs(x));
    else
      h=ef^(1/2);
    end;
    temp=x+h;
    h=temp-x;
    farg2=gamma;
    farg2(j)=gamma(j)+h;
    vsn=getSandVnew(farg2, beta, alpha_g, alpha_b, phi);
    sn=vsn(:,dimparm+1); 
    score(i,j)=(s(i,1)-sn(i,1))/(h); 
    end;

    %%%%%%%%%%%%% numerical derivative wrt betas

    for j=1:dimbeta         

    x=beta(j);
    if (x~=0)
      h=ef^(1/2)*(1+abs(x));
    else
      h=ef^(1/2);
    end;
    temp=x+h;
    h=temp-x;
    farg2=beta;
    farg2(j)=beta(j)+h;
    vsn=getSandVnew(gamma, farg2, alpha_g, alpha_b, phi);
    sn=vsn(:,dimparm+1); 
    score(i,dimgamma+j)=(s(i,1)-sn(i,1))/(h);   
    end;
    
    %%%%%%%%%%%%% numerical derivative wrt alpha_gamma
    
    if (~strcmp(MIXPCORSTR,'indep'))
    for j=1:dimalpha_g
    x=alpha_g(j);
    if (x~=0)
      h=ef^(1/2)*(1+abs(x));
    else
      h=ef^(1/2);
    end;
    temp=x+h;
    h=temp-x;
    farg2=alpha_g;
    farg2(j)=alpha_g(j)-h;
    vsn=getSandVnew(gamma, beta, farg2, alpha_b, phi); 
    sn=vsn(:,dimparm+1); 
    score(i,dimgamma+dimbeta+j)=(s(i,1)-sn(i,1))/(h);
    end;
    end;   % for if

   %%%%%%%%%%%%% numerical derivative wrt alpha_beta
   if (~strcmp(CORSTRUCT,'indep'))

   for j=1:dimalpha_b
    x=alpha_b(j);
    if (x~=0)
      h=ef^(1/2)*(1+abs(x));
    else
      h=ef^(1/2);
    end;
    temp=x+h;
    h=temp-x;
    farg2=alpha_b;
    farg2(j)=alpha_b(j)+h;
    vsn=getSandVnew(gamma, beta, alpha_g, farg2, phi);
    sn=vsn(:,dimparm+1);
    score(i,dimgamma+dimbeta+dimalpha_g+j)=(s(i,1)-sn(i,1))/(h);
   end; 
   end;
     
    %%%%%%%%%%%%% numerical derivative wrt phi
    x=phi;
   if (x~=0)
       h=ef^(1/2)*(1+abs(x));
    else
      h=ef^(1/2);
    end;
    temp=x+h;
    h=temp-x;  
    farg2=phi+h;
    vsn=getSandVnew(gamma, beta, alpha_g, alpha_b, farg2);
    sn=vsn(:,dimparm+1);
    score(i,end)=(sn(i,1)-s(i,1))/(h);
          
end;  % for i   

