/* SAS code for ZIP and ZINB models in analyzing appleroot data*/
/*for Table 2 in the paper*/

options ls=78 ps=65;
options formdlim="*";

data appleroot;
  infile 'appleroot.dat';
  input photo BAP Y ;
  one=1;
  y0=(y=0);
  if BAP=1 then BAP=2.2;
  if BAP=2 then BAP=4.4;
  if BAP=3 then BAP=8.8;
  if BAP=4 then BAP=17.6;
  IB1=(photo=8);
  IB2=(photo=16);
  photoc=(photo=16);
  log_BAP=log(BAP); 
  run;


/*ZIP*/
proc nlmixed data=appleroot;

parms b1=1.79 b2=2 b3=.09 b4=-.17 g1=-4.26 g2=-.1;

etab=b1*IB1+b2*IB2 +b3*log_BAP*IB1+b4*log_BAP*IB2;
etag=g1*IB1 +g2*IB2;

/*Define loglikelihood function of ZIP model under independence*/ 

if (y0=1) then ll=log(exp(etag)+exp(-exp(etab)))-log(1+exp(etag));
  else ll=(y*etab-exp(etab)) -log(1+exp(etag))-lgamma(y+1);

  model y~general(ll);
run;


/*ZINB*/
proc nlmixed data=appleroot;

parms b1=1.79 b2=2 b3=.09 b4=-.17 g1=-4.26 g2=-.1 alpha=1.5;

etab=b1*IB1+b2*IB2 +b3*log_BAP*IB1+b4*log_BAP*IB2;
etag=g1*IB1 +g2*IB2;

/*Define loglikelihood function of ZINB model under independence*/ 

  if (y0=1) then ll=log(exp(etag)+(1+alpha*exp(etab))**(-1/alpha))-log(1+exp(etag));
  else ll=-log(1+exp(etag))+lgamma(y+1/alpha)-lgamma(y+1)-lgamma(1/alpha)
-(1/alpha)*log(1+alpha*exp(etab))-y*log(1+1/(alpha*exp(etab)));

  model y~general(ll);
run;

  
